import json
from collections import defaultdict

def select_and_transform_data(input_file, output_file, samples_per_category=4, total_samples=250):
    """
    读取jsonl文件，按answer分类，每类选指定条数数据，转换字段并保存
    
    :param input_file: 输入jsonl文件路径
    :param output_file: 输出jsonl文件路径
    :param samples_per_category: 每类选取的样本数
    :param total_samples: 总共需要选取的样本数
    """
    # 按answer分类存储数据
    categorized_data = defaultdict(list)
    
    # 1. 读取并分类数据
    with open(input_file, 'r', encoding='utf-8') as f:
        for line in f:
            try:
                data = json.loads(line.strip())
                answer = data.get('answer')
                if answer:
                    categorized_data[answer].append(data)
            except json.JSONDecodeError:
                print(f"警告：跳过无法解析的行: {line}")
                continue
    
    # 2. 从每类中随机选取指定数量的样本
    selected_data = []
    selected_count = 0
    
    for category, items in categorized_data.items():
        # 计算当前类别最多能取多少样本（避免超过总数限制）
        remaining = total_samples - selected_count
        take = min(samples_per_category, len(items), remaining)
        
        if take <= 0:
            break  # 已达到总数要求
        
        # 随机选取（这里简单取前take条，如需随机可改为random.sample）
        selected = items[:take]
        
        # 3. 转换字段并添加到结果
        for item in selected:
            transformed = {
                "工单编号": item["uid"],
                "data": item["data"],
                "answer": item["answer"]
            }
            selected_data.append(transformed)
            selected_count += 1
            
            if selected_count >= total_samples:
                break
    
    # 4. 保存结果到jsonl文件
    with open(output_file, 'w', encoding='utf-8') as f:
        for item in selected_data:
            f.write(json.dumps(item, ensure_ascii=False) + '\n')
    
    print(f"处理完成！共选取{selected_count}条数据，保存到{output_file}")
    print(f"分类统计：{ {k: len([d for d in selected_data if d['answer'] == k]) for k in categorized_data} }")

# 使用示例
input_file = "/data/zhaochsh01/buquan/12345/count_result/12345_prompt.jsonl"  # 替换为你的输入文件路径
output_file = "output_prompt.jsonl"  # 输出文件路径
select_and_transform_data(input_file, output_file, samples_per_category=4, total_samples=250)