import json
from vllm import LLM, SamplingParams
from vllm.transformers_utils.tokenizer import get_tokenizer
from tqdm import tqdm, trange
from copy import deepcopy

# need vllm==0.4.2
# firefly_copy 使用

# step1 读入待跑 tag 的 jsonl 文件
input_jsonl_file = "/data/zhaochsh01/buquan/12345/tool/output.jsonl"
with open(input_jsonl_file, 'r', encoding='utf8') as f:
    raw_data_list = f.readlines()
print("len_raw_data_list:", len(raw_data_list))

# 这里测试,先跑前100个数据
# raw_data_list = raw_data_list[:100]

# step2 读取 instag 模型 配置 tokenizer
model_path = '/model-pvc/instagger_qwen1_8B/'
tokenizer = get_tokenizer(model_path, trust_remote_code=True)
tokenizer.chat_template = (
    "{% for message in messages %}"
    "{{ '### ' + message['role'].capitalize() + ':\\n' }}"
    "{{ message['content'] | string + '\\n' }}"
    "{% endfor %}"
    "{% if add_generation_prompt %}"
    "{{ '### Assistant:\\n' }}"
    "{% endif %}"

)
# step3 这里内存够用，一次性读取所有文件进行拼接 prompt 处理
prompt_text_list = []
for index, line in enumerate(raw_data_list):
    # print("process:", index)
    json_obj = json.loads(line)
    query = json_obj["主要内容"]
    prompt = query

    messages = [
        {"role": "system", "content": "You are an intelligent assistant, please identify the category based on the user’s query and return the corresponding category."},
        {"role": "user", "content": prompt}
    ]
    text = tokenizer.apply_chat_template(
        messages,
        tokenize=False,
        add_generation_prompt=True
    )
    prompt_text_list.append(text)

print("len_prompt_text_list:", len(prompt_text_list))
assert len(raw_data_list) == len(prompt_text_list)


def extract_raw_qwen_res(response):
    if "<|im_end|>" in response:
        response = response.split("<|im_end|>")[0]
    return response


# step4: vllm 推理参数设置,官方推荐这里要求 temperature=0
sampling_params = SamplingParams(max_tokens=64,
                                 temperature=0,
                                 top_p=0.8,
                                 repetition_penalty=1.05)

# step5: 读入模型 这里读入模型往后放，数据有问题在load 模型之前暴露出来节省时间
# set gpu_memory_utilization=0.95 for OOM error
vllm_model = LLM(model=model_path,
                 tensor_parallel_size=1,
                 gpu_memory_utilization=0.85,
                 max_num_seqs=64,               # 减少最大并发序列数（默认256）
                 max_model_len=2048,            # 减少最大上下文长度
                 enforce_eager=True, 
                 trust_remote_code=True)
vllm_model.set_tokenizer(tokenizer)

# step6： 分批量推理并且得到结果的过程
tag_result_list = []
batch_size = 1024
for start in trange(0, len(prompt_text_list), batch_size):
    print("index:", start)
    batch_list = prompt_text_list[start: start + batch_size]
    vllm_outputs = vllm_model.generate(batch_list, sampling_params)
    for output in vllm_outputs:
        prompt = output.prompt
        cur_tag_result = extract_raw_qwen_res(output.outputs[0].text)
        tag_result_list.append(cur_tag_result)

print("len_tag_result_list:", len(tag_result_list))

assert len(tag_result_list) == len(prompt_text_list)

# step7: 写入结果
print("开始写 instag 结果 数据")
single_turn_instag_path = "./12345/instag/123451wfilter_instag.jsonl"
print("single_turn_instag_path:", single_turn_instag_path)
single_f_sample_save = open(single_turn_instag_path, "w")
write_single_sample_count = 0
for line, tag in zip(raw_data_list, tag_result_list):
    json_line = json.loads(line)
    json_line["ins_tag_label"] = tag
    cur_final_line = json.dumps(deepcopy(json_line), ensure_ascii=False)
    write_single_sample_count = write_single_sample_count + 1
    single_f_sample_save.write(cur_final_line + "\n")
    if write_single_sample_count % 100000 == 0:
        print("write_single_count:", write_single_sample_count)

print("all finish success !")
